<?php

namespace Drupal\commerce_multisafepay_recurring\EventSubscriber;

use Drupal\commerce_recurring\RecurringOrderManagerInterface;
use Drupal\Core\Entity\EntityTypeManagerInterface;
use Drupal\state_machine\Event\WorkflowTransitionEvent;
use Symfony\Component\EventDispatcher\EventSubscriberInterface;
use Drupal\commerce_multisafepay_payments\API\Client;
use Drupal\commerce_multisafepay_payments\Helpers\ApiHelper;
use Drupal\commerce_multisafepay_payments\Helpers\GatewayHelper;


class OrderSubscriber implements EventSubscriberInterface {

  /**
   * {@inheritdoc}
   */
  public static function getSubscribedEvents() {
    return [
      'commerce_order.place.post_transition' => ['onPlace'],
    ];
  }

  /**
   * Starts a billing cycle for recurring msp.
   *
   * @param \Drupal\state_machine\Event\WorkflowTransitionEvent $event
   *   The transition event.
   */
  public function onPlace(WorkflowTransitionEvent $event) {
    /** @var \Drupal\commerce_order\Entity\OrderInterface $order */
	//$subscription_storage = $this->entityTypeManager->getStorage('commerce_subscription');
    $order = $event->getEntity();
	$paymentGateway=$order->get('payment_gateway');
	/*track this into a custom table*/
	$connection = \Drupal::service('database');
	$mspGatewayHelper = new GatewayHelper();
	$gatewayPluginID='';
	$paymentGateway='';
	
	
	/*end*/
//echo"<pre>";
if ($order->bundle() == 'recurring') {


    foreach ($order->getItems() as $order_item) 
	{
		$purchased_entity = $order_item->getPurchasedEntity();
		
		$orderid='';
		 $subscriptionEntiy = $order_item->get('subscription')->entity;
		$subscription_id=$order_item->get('subscription')->target_id;
		$initial_order_id=$subscriptionEntiy->get('initial_order')->target_id;
		
		/*Load initial order to confirm the attached MSP-PluginID*/

		$initialOrderObject=\Drupal\commerce_order\Entity\Order::load($initial_order_id);
		$initialGateway = $initialOrderObject->get('payment_gateway')->first()->entity;
		if(!empty($initialGateway)){ 
			
			$gatewayPluginID=$initialGateway->getPluginId();
			$paymentGateway=$initialOrderObject->get('payment_gateway')->first()->entity->id();
		
		}	
		
		/*end*/


		$purchasedID = $order_item->get('purchased_entity')[0]->get('target_id')->getValue();
		$next_order_no_id = $order_item->get('order_item_id')[0]->get('value')->getValue();
		$order_id=$order_item->get('order_id')[0]->get('target_id')->getValue();
	
		

		/*Check the linked payment Gateway from Initial Order*/
		$paymentGateway=$connection->select('commerce_order', 'commerce_order')
		->condition('commerce_order.order_id', $initial_order_id, '=')
		->fields('commerce_order', ['payment_gateway'])
		->execute()->fetchField();;
		/*End*/
	
	//process msp-recurring if initital transaction linked with MSP-Methods
	if ($mspGatewayHelper->isMspGateway($gatewayPluginID) && $order_id!='') 
	{

	/*$connection->insert('trackdata')
  ->fields([
    'trackdata' => "Inside of msprecurring function",
    'actiondate' => \Drupal::time()->getRequestTime(),
    ])
  ->execute();*/

	$mspRecurring=$this->processMultisafePayRecurring($initial_order_id,$order_id);
	if(isset($mspRecurring))
		{
		
		/*Update the status of recurring order */
		//$latest_order = \Drupal\commerce_order\Entity\Order::load($order_id);
		//$latest_order->set('state', 'completed');
		//$latest_order->set('payment_gateway',$orderGateway);
		//$latest_order->save();
		/*end*/
			
		$connection->insert('commerce_multisafepay_recurring_orders')
			->fields([
			'order_id' =>$order_id,
			'msprecurring_payment_status' =>'completed',
			'msp_paymentgateway'=>$paymentGateway,
			])
			->execute();
		 
		 
		 \Drupal::logger('commerce_multisafepay_recurring')->notice('Recurring payment successfully done!'.$order_id);
		}else{
			 \Drupal::logger('commerce_multisafepay_recurring')->notice('Process failed corresponding to Order!'.$order_id);
		}
	 }
  }
 } 
}


  public function processMultisafePayRecurring($initialorderId='',$currentOrderID=''){
  
  $client = new Client();

  $connection = \Drupal::service('database');
$mspApiHelper = new ApiHelper();


$mspApiHelper->setApiSettings($client, 'test');
/*program for retrive a recurring-id from MultisafePay via API-https://docs.multisafepay.com/api/#retrieve-an-order */

//get the orderid in which recurring-payments enabled

if(isset($initialorderId) && $initialorderId!=''){


$msp_orderid=$initialorderId;

$mspOrder = $client->orders->get('orders', $msp_orderid);
if(!empty($mspOrder) && isset($mspOrder->payment_details)){
	$recurring_id = $mspOrder->payment_details->recurring_id;

	
	/*process the recurring payment corresponding the retreived Recurring ID*/
	$recurringOrder=array(
          "type" => 'direct',
          "order_id" =>($currentOrderID) ? $currentOrderID : time(),
          "currency" => $mspOrder->currency,
          "amount" => $mspOrder->amount,
          "description" => "This is a recurring payment corresponding to order:".$msp_orderid,
          "gateway" => $mspOrder->payment_details->type,
          "recurring_id" => $recurring_id, 
          "days_active" => "30",
      );
	$recurringorderstatus = $client->orders->post($recurringOrder);
	return $recurringorderstatus;
	/*End*/

}

}
  
  }

}